Attribute VB_Name = "modListSysColoors"
Option Explicit

'APIs To "OwnerDraw" A ListBox, And Override Any Of The ListBox's Events

'Describes the width, height, and location of a rectangle.
Public Type RECT
        Left As Long
        Top As Long
        Right As Long
        Bottom As Long
End Type

'Provides necessary information the owner window to determine how to paint an owner-drawn control or menu item.
Public Type DRAWITEMSTRUCT
        CtlType As Long
        CtlID As Long
        itemID As Long
        itemAction As Long
        itemState As Long
        hwndItem As Long
        hdc As Long
        rcItem As RECT
        itemData As Long
End Type

'Copies a block of memory from one location to another
Public Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)

'The SetWindowLong function changes an attribute of the specified window.
Public Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long

'The CallWindowProc function passes message information to the specified window procedure.
Public Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hwnd As Long, ByVal Msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long

'Sends the specified message to a window or windows
Public Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long

'This function creates a logical brush that has the specified solid color
Public Declare Function CreateSolidBrush Lib "gdi32" (ByVal crColor As Long) As Long

'This function fills a rectangle using the specified brush
Public Declare Function FillRect Lib "user32" (ByVal hdc As Long, lpRect As RECT, ByVal hBrush As Long) As Long

'This function deletes a logical pen, brush, font, bitmap, region, or palette, freeing all system resources associated with the object
Public Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long

'The SetBkColor function sets the current background color to the specified color value
Public Declare Function SetBkColor Lib "gdi32" (ByVal hdc As Long, ByVal crColor As Long) As Long

'The SetTextColor function sets the text color for the specified device context to the specified color.
Public Declare Function SetTextColor Lib "gdi32" (ByVal hdc As Long, ByVal crColor As Long) As Long

'The TextOut function writes a character string at the specified location, using the currently selected font, background color, and text color
Public Declare Function TextOut Lib "gdi32" Alias "TextOutA" (ByVal hdc As Long, ByVal x As Long, ByVal y As Long, ByVal lpString As String, ByVal nCount As Long) As Long

'Draws a rectangle in the style used to indicate that the rectangle has the focus
Public Declare Function DrawFocusRect Lib "user32" (ByVal hdc As Long, lpRect As RECT) As Long

'Retrieves the current color of the specified display element
Public Declare Function GetSysColor Lib "user32" (ByVal nIndex As Long) As Long

Public Const LB_GETTEXT = &H189 'Gets a string from a list box
Public Const WM_DRAWITEM = &H2B 'Sent to the parent window of an owner-drawn button, combo box, list box, or menu when a visual aspect of the button, combo box, list box, or menu has changed.
Public Const GWL_WNDPROC = (-4) 'Sets a new address for the window procedure
Public Const ODS_FOCUS = &H10
Public Const ODT_LISTBOX = 2

Public lPrevWndProc As Long

'Function To Override ListBox Colours
'hwnd = ListBox Window Handle
'Msg = Message To Send
'wParam & lparam = What Message to Send

Public Function OverrideList(ByVal hwnd As Long, ByVal Msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    Dim tItem As DRAWITEMSTRUCT
    Dim sBuff As String * 255
    Dim sItem As String
    Dim lBack As Long

    If Msg = WM_DRAWITEM Then
        'Redraw ListBox
         Call CopyMemory(tItem, ByVal lParam, Len(tItem))

        'Is The Control Indeed A ListBox 
        If tItem.CtlType = ODT_LISTBOX Then
            'Get Current Item's Text
            Call SendMessage(tItem.hwndItem, LB_GETTEXT, tItem.itemID, ByVal sBuff)
            sItem = Left(sBuff, InStr(sBuff, Chr(0)) - 1)

            If (tItem.itemState And ODS_FOCUS) Then
                'If Item Has Focus - Highlight it
                lBack = CreateSolidBrush(GetSysColor(COLOR_HIGHLIGHT))

               'Fill Item's Rect
                Call FillRect(tItem.hdc, tItem.rcItem, lBack)
                'BackColour Of Item
                Call SetBkColor(tItem.hdc, GetSysColor(COLOR_HIGHLIGHT))

                'TextColour Of Item
                Call SetTextColor(tItem.hdc, GetSysColor(COLOR_HIGHLIGHTTEXT))

                'Draw The Text
                TextOut tItem.hdc, tItem.rcItem.Left, tItem.rcItem.Top, ByVal sItem, Len(sItem)
                DrawFocusRect tItem.hdc, tItem.rcItem
            Else
                'Item Doesn't Have Focus - Draw it's Coloured Background
                'Use Colour From ItemData
                lBack = CreateSolidBrush(tItem.itemData)

                'Fill Item's Rect
                Call FillRect(tItem.hdc, tItem.rcItem, lBack)

                'BackColour Of Item
                Call SetBkColor(tItem.hdc, tItem.itemData)

                'TextColour Of Item
                Call SetTextColor(tItem.hdc, IIf(tItem.itemData = vbBlack, vbWhite, vbBlack))

                'Draw The Text
                TextOut tItem.hdc, tItem.rcItem.Left, tItem.rcItem.Top, ByVal sItem, Len(sItem)
            End If
            Call DeleteObject(lBack)

            'Don't Pass a Value
            OverrideList = 0
            Exit Function

        End If

    End If

    OverrideList = CallWindowProc(lPrevWndProc, hwnd, Msg, wParam, lParam)
End Function
